// C Source File
// Created 09.11.2003; 16:37:54

//                             |||
//                            (o o)
//              +----------oo0-(_)-0oo------------------------------------------+
//              |																																|
//              |	   	    Filename:	mandelbrot.c	        											|
//              |		    					                         											|
//              |		    Coded by:	David Hasenfratz					  								  |
//							|				Mail: david_hasenfratz@yahoo.de                         |
//              |	    																													|
//              +---------------------------------------------------------------+

// This program draws the mandelbrot set
// Information what mandelbrot set is:
// http://www.students.tut.fi/~warp/Mandelbrot/

// Usage: mandelbr(xMin, xMax, yMax, nums of interations)
// Bsp: mandelbr(-2.0, 1.5, 1.1, 15) 

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

//#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

// Draw function
void drawMandelbrot(float xMin, float xMax, float yMax, int recursiv); 

// Main Function
void _main(void)
{
	void *savescreen = NULL;

  if (!(savescreen = malloc(LCD_SIZE+LCD_SIZE+LCD_SIZE))) {
     ST_helpMsg("ERROR: Not enough memory!");
       return;
  }
  
  //Activate grayscale mode
  if (!GrayOn ())
  {
  	ST_helpMsg("ERROR: Could not switch to grayscale mode!");
  	return;
	}
	
  LCD_save(savescreen);
	
	// Get the arguments
	ESI argptr = top_estack;
  float xMin = GetFloatArg (argptr);
  float xMax = GetFloatArg (argptr);
  float yMax = GetFloatArg (argptr);
  int recursiv = GetIntArg (argptr);
  while (GetArgType (top_estack) != END_TAG)  // Clean up arguments
  top_estack = next_expression_index (top_estack);
  top_estack--;
	
	clrscr();									// Clear screen
	
	int usage = 0;
	
	// If recursiv is less than 0, usage from the user is unknown
	if(recursiv <= 0)
	{
		usage = 1;
	}

  if(usage == 0)
  {
  	// Draw the mandelbrot set
  	drawMandelbrot(xMin, xMax, yMax, recursiv);
  
    ngetchx ();
    GrayOff();
  
  	// Restore screen
  	LCD_restore(savescreen);
  	free(savescreen);
  	// Message
		ST_helpMsg("by David Hasenfratz");
	}
	else
	{
		GrayOff();
		
		// Restore screen
		LCD_restore(savescreen);
  	free(savescreen);
  	// Message
		ST_helpMsg("Use: mandelbr(xMin, xMax, yMax, nums)");		
	}	
}

void drawMandelbrot(float xMin, float xMax, float yMax, int recursiv)
{
	short exit = 0;

	// 1 pixel has the value delta
	float delta = (xMax-xMin)/LCD_WIDTH;
  
  // The calculated yMax
  //yMax = (delta*LCD_HEIGHT)/2;
  
  float zx, zy, tx, ty, cx, cy;
	int y,x,i;
	
	// For each pixel from the top to the buttom
	for(y = 0; y <= LCD_HEIGHT; y++)
	{
		if(exit == 1)
			break;
			
		// For each pixel from left to right
		for(x = 0; x <= LCD_WIDTH; x++)
		{
			if(exit == 1)
				break;
			
			zx = 0;
			zy = 0;
			
			cx = xMin+x*delta;
			cy = yMax-y*delta;
			
			for(i = 0; i < recursiv; i++)
			{
				// See if user pressed ESCAPE
				BEGIN_KEYTEST
				if(_keytest_optimized (RR_ESC))
					exit = 1;
				END_KEYTEST	
					
				// Calculate the new zx, zy
				tx = zx*zx-zy*zy+cx;
				ty = 2*zx*zy+cy;
				
				zx = tx;
				zy = ty;
				
				if(zx*zx+zy*zy>4)
					break;	
			}
			// Draw the point if it is in the mandelbrot set
			if(zx*zx+zy*zy<4)
			{	
				SetPlane(DARK_PLANE);
				DrawPix(x, y, A_NORMAL);
			}
			// Draw lightgrey is its near to the mandelbrot set
			if(i > recursiv*0.4)
			{
				SetPlane(LIGHT_PLANE);
				DrawPix(x, y, A_NORMAL);
			}
		}
	}
}